# Copyright (c) 2017 Civic Knowledge. This file is licensed under the terms of the
# Revised BSD License, included in this distribution as LICENSE

"""
CLI program for storing packages to Data.World
"""

# flake8: noqa

import json
import mimetypes
import sys
from os import getcwd
from os.path import basename, join

from metapack import open_package
from metapack.cli.core import err, prt, warn
from metatab import (DEFAULT_METATAB_FILE, MetatabDoc, MetatabError,
                     resolve_package_metadata_url)
from metatab.util import slugify
from rowgenerators import Url, get_cache

try:
    import datadotworld as dw
    from datadotworld.client.api import RestApiError
except ImportError:
    err(
        "To run the Metataworld importer, you must first install the datadotworld package. See https://github.com/datadotworld/data.world-py")


def metaworld():
    import argparse
    parser = argparse.ArgumentParser(
        prog='metakan',
        description='Publish packages to Data.World')

    parser.add_argument('-i', '--info', default=False, action='store_true',
                        help="Show package information")

    parser.add_argument('metatabfile', nargs='?', default=DEFAULT_METATAB_FILE, help='Path to a Metatab file')

    class MetapackCliMemo(object):
        def __init__(self, args):
            self.cwd = getcwd()
            self.args = args
            self.cache = get_cache('metapack')

            self.mtfile_arg = args.metatabfile if args.metatabfile else join(self.cwd, DEFAULT_METATAB_FILE)

            self.mtfile_url = Url(self.mtfile_arg)
            self.resource = self.mtfile_url.parts.fragment

            self.package_url, self.mt_file = resolve_package_metadata_url(self.mtfile_url.rebuild_url(False, False))

    m = MetapackCliMemo(parser.parse_args(sys.argv[1:]))

    try:
        doc = MetatabDoc(m.mt_file, cache=m.cache)
    except (IOError, MetatabError) as e:
        err("Failed to open metatab '{}': {}".format(m.mt_file, e))

    if m.args.info:
        package_info(doc)
    else:
        send_to_dw(doc)

    exit(0)


def package_info(doc):
    client = dw.api_client()

    username = 'ericbusboom'

    title = doc.find_first_value("Root.Title")
    key = join(username, slugify(title))

    try:
        ds = client.get_dataset(key)
        prt(json.dumps(ds, indent=4))
    except RestApiError as e:
        err(e)


def get_resource_urls(doc):
    resources = {}

    for dist in doc.find("Root.Distribution"):

        try:
            package_url, metadata_url = resolve_package_metadata_url(dist.value)
        except Exception as e:
            warn("Failed for Distribution {}; {}".format(dist.value, e))
            continue

        u = Url(package_url)

        if u.resource_format == 'zip':
            prt("Skipping ZIP package ", package_url)

        elif u.resource_format == 'xlsx':
            if False:
                resources[basename(package_url)] = package_url
                prt("Adding XLS package ", package_url)
                pass

        elif u.resource_format == 'csv':

            resources[basename(package_url)] = u.signed_resource_url

            prt("Adding CSV package {}".format(basename(package_url)))

            try:
                p = open_package(package_url)
            except (IOError, MetatabError) as e:
                err("Failed to open package '{}' from reference '{}': {}".format(package_url, dist.value, e))

            for r in p.resources():

                mimetype = mimetypes.guess_type(r.resolved_url)[0]

                try:
                    ext = mimetypes.guess_extension(mimetype)[1:]
                except:
                    ext = None

                # '.csv': Data>world currently get the format from the name, not the URL
                resources[r.name + '.csv'] = r.resolved_url
                prt("Adding CSV resource {}".format(r.name))
        else:
            prt('Skipping {}'.format(package_url))

    return resources


def truncate(v, l, suffix=''):
    return v[:(l - len(suffix))] if len(v) > l else v


def send_to_dw(doc):
    client = dw.api_client()

    username = 'ericbusboom'

    title = doc.find_first_value("Root.Title")
    key = username + '/' + slugify(truncate(title, 30))

    d = dict(
        title=truncate(title, 30),
        description=doc.find_first_value("Root.Description"),
        summary=doc.markdown,
        visibility='OPEN',
        files=get_resource_urls(doc)
    )

    try:

        ds = client.get_dataset(key)  # Raise an error if the dataset does not exist

        ds = client.replace_dataset(key, **d)

        ds = client.get_dataset(key)

    except RestApiError:

        ds = client.create_dataset(username, **d)

        ds = client.get_dataset(key)



# Copyright (c) 2017 Civic Knowledge. This file is licensed under the terms of the
# Revised BSD License, included in this distribution as LICENSE

"""
CLI program for storing pacakges in CKAN

The program uses the Root.Distributions in the source package to locate packages to link into a CKAN record.

"""

import mimetypes
import traceback
from os import getenv
from os.path import basename
from textwrap import dedent

from metapack import Downloader, MetapackDoc, open_package
from metapack.cli.core import err, prt, warn, write_doc
from metapack_build.build import find_csv_packages
from metapack_build.package.s3 import S3Bucket
from metatab import MetatabError

from metapack.cli.core import MetapackCliMemo as _MetapackCliMemo

downloader = Downloader.get_instance()

class MetapackCliMemo(_MetapackCliMemo):

    def __init__(self, args, downloader):
        super().__init__(args, downloader)

        self.api_key = self.args.api or getenv('METAKAN_API_KEY')

        self.ckan_url = self.args.ckan or getenv('METAKAN_CKAN_URL')

        if not self.ckan_url:
            err("Set the --ckan option or the METAKAN_CKAN_URL env var to set the URL of a ckan instance")

        if not self.api_key:
            err("Set the --api option METAKAN_API_KEY env var  with the API key to a CKAN instance")

    def update_mt_arg(self, metatabfile):
        """Return a new memo with a new metatabfile argument"""
        o = MetapackCliMemo(self.args)
        return o

def metakan(subparsers):


    parser = subparsers.add_parser(
        'ckan',
        help='CKAN management of Metatab packages'
    )

    parser.set_defaults(run_command=run_ckan)

    parser.add_argument('-i', '--info', default=False, action='store_true',
                   help="Show configuration information")

    parser.add_argument('-c', '--ckan', help="URL for CKAN instance")

    parser.add_argument('-a', '--api', help="CKAN API Key")

    parser.add_argument('-g', '--group', nargs='?', action='append', help="Assign an additional group. Can be used "
                                                                          "multiple times, one onece with values "
                                                                          "seperated by commas")

    parser.add_argument('-t', '--tag', nargs='?', action='append', help="Assign an additional tag. Can be used "
                                                                        "multiple timesm or once with values "
                                                                        "seperated by commas")

    parser.add_argument('-p', '--packages', action='store_true',
                        help="The file argument is a text file with a list of package URLs to load")

    parser.add_argument('-C', '--configure', action='store_true',
                        help="Load groups and organizations from a metatab file")

    parser.add_argument('-D', '--dump', action='store_true',
                        help="Dump groups and organizations to a metatab file")

    parser.add_argument('metatabfile', nargs='?',
                        help='Path to a Metatab file, or an s3 link to a bucket with Metatab files. ')


def run_ckan(args):

    m = MetapackCliMemo(args, downloader=downloader)


    if m.mtfile_url.scheme == 's3':
        # Find all of the top level CSV files in a bucket and use them to create CKan entries

        b = S3Bucket(m.mtfile_arg)

        for e in b.list():
            key = e['Key']
            if '/' not in key and key.endswith('.csv'):
                url = b.access_url(key)
                prt("Processing", url)
                send_to_ckan(m.update_mt_arg(url))

    elif m.args.packages:
        # Load a list of packages
        with open(m.mtfile_arg) as f:
            for line in f.readlines():
                url = line.strip()
                prt("Processing", url)
                try:
                    send_to_ckan(m.update_mt_arg(url))
                except Exception as e:
                    warn("Failed to process {}: {}".format(line, e))

    elif m.args.configure:
        configure_ckan(m)

    elif m.args.dump:
        dump_ckan(m)

    else:
        send_to_ckan(m)

    exit(0)

def send_to_ckan(m):

    from ckanapi import RemoteCKAN, NotFound
    try:
        doc = MetapackDoc(m.mt_file, cache=m.cache)

    except (IOError, MetatabError) as e:
        err("Failed to open metatab '{}': {}".format(m.mt_file, e))

    # Check for distributions, if there aren't any, try to find a CSV package,
    # which may be an S3 package with distributions.

    if not doc.find('Root.Distribution'):
        doc = find_csv_packages(m, downloader)
        if doc:
            prt("Orig doc has no distributions, switching to doc from built packages:")
            prt("    ", doc.ref)

    if not doc or not doc.find('Root.Distribution'):
        err("No distributions found. Giving up")


    c = RemoteCKAN(m.ckan_url, apikey=m.api_key)

    ckanid = doc.find_first_value('Root.Ckanid')

    unversioned_name = doc.as_version(None)
    ckan_name = unversioned_name.replace('.','-')

    id_name = ckanid or ckan_name

    try:
        pkg = c.action.package_show(name_or_id=id_name)
        prt("Updating CKAN dataset for '{}'".format(id_name))
        found = True
    except NotFound as e:
        e.__traceback__ = None
        traceback.clear_frames(e.__traceback__)
        found = False

    if not found:
        try:
            pkg = c.action.package_show(name_or_id=ckan_name)
            prt("Updating CKAN dataset for '{}'".format(id_name))
            found = True
        except NotFound as e:
            e.__traceback__ = None
            traceback.clear_frames(e.__traceback__)
            found = False

    if not found:
        try:
            pkg = c.action.package_create(name=ckan_name)
        except Exception as e:
            err("Failed to create package for name '{}': {} ".format(ckan_name, e))

        prt("Adding CKAN dataset for '{}'".format(ckan_name))

    pkg['title'] = doc.find_first_value('Root.Title')

    if not pkg['title']:
        pkg['title'] = doc.find_first_value('Root.Description')

    pkg['version'] =  doc.find_first_value('Root.Version')

    pkg['groups'] = [ {'name': g.value } for g in doc['Root'].find('Root.Group')]

    if m.args.group:

        for g in m.args.group:
            if ',' in g:
                for g_ in g.split(','):
                    pkg['groups'].append({'name': g_})
            else:
                pkg['groups'].append({'name': g })

    pkg['tags'] = [{'name': g.value} for g in doc['Root'].find('Root.Tag')]

    if m.args.tag:
        for g in m.args.tag:
            if ',' in g:
                for g_ in g.split(','):
                    pkg['tags'].append({'name': g_})
            else:
                pkg['tags'].append({'name': g })

    org_name = doc.get_value('Root.Origin', doc.get_value('Root.CkanOrg'))

    if org_name:
        org_name_slug = org_name.replace('.','-')
        try:

            owner_org = c.action.organization_show(id=org_name_slug).get('id')
            pkg['owner_org'] = owner_org
        except NotFound:
            warn("Didn't find org for '{}'; not setting organization ".format(org_name_slug))
            org_name_slug = None
    else:
        org_name_slug = None

    extras = {}

    for t in doc.find('*.*', section='Root'):
        if not t.term_is('Root.Distribution'):
            extras[t.qualified_term] = t.value

    pkg['extras'] = [ {'key':k, 'value':v} for k, v in extras.items() ]

    resources = []

    # Try to set the markdown from a CSV package, since it will have the
    # correct links.
    markdown = ''

    inst_distributions = []
    csv_package = None


    def add_package(format, resources):

        package_url = dist.package_url

        d = dict(
            url=str(package_url.inner),
            name=basename(package_url.path),
            format=format,
            mimetype=mimetypes.guess_type(package_url.path)[0],
            description=load_instructions_description('{} version of package, in Metatab format.'.format(format),
                                                      str(package_url.inner))
        )
        resources.append(d)
        prt("Adding {} package {}".format(d['format'], d['name']))


    def load_resources(dist_url, package_url, metadata_url):

        try:
            p = open_package(metadata_url)
        except (IOError, MetatabError) as e:
            err("Failed to open package '{}' from reference '{}': {}".format(package_url, dist_url, e))

        for r in p.resources():

            mimetype = mimetypes.guess_type(r.resolved_url.path)[0]

            try:
                ext = mimetypes.guess_extension(mimetype)[1:]
            except:
                ext = None

            d = dict(
                name=r.name,
                format=ext,
                url=str(r.resolved_url),
                mimetype=mimetype,
                description=r.markdown
            )

            resources.append(d)
            prt("  Adding resource {}".format(d['name']))

        return p


    prt("Distributions:")
    for dist in doc.find('Root.Distribution'):
        prt("    {}".format(dist.value))


    for dist in doc.find('Root.Distribution'):

        if dist.type == 'zip':
            add_package('ZIP', resources)
            inst_distributions.append(dist)

        elif dist.type == 'xlsx':
            add_package('XLSX', resources)

        elif dist.type == 'csv':
            add_package('CSV', resources)
            inst_distributions.append(dist)

            # Resources are always created from the CSV package.
            csv_package = load_resources(dist.url, dist.package_url, dist.metadata_url)

        elif dist.type == 'fs':
            pass

        else:
            warn("Unknown distribution type '{}' for '{}'  ".format(dist.type, dist.value))

    from rowgenerators.appurl.web import WebUrl
    if isinstance( m.mt_file.inner, WebUrl) and  m.mt_file.inner.target_format == 'csv':

        dist = m.doc['Root'].new_term('Root.Distribution', m.mt_file)
        inst_distributions.append(dist)
        csv_package = load_resources(dist.url, dist.package_url, dist.metadata_url)


    if not csv_package:
        err("Didn't find a CSV package. Giving up")

    markdown = csv_package.markdown + "\n\n"+ package_load_instructions(inst_distributions)

    try:
        pkg['notes'] = markdown or doc.markdown #doc.find_first_value('Root.Description')
    except (OSError, DownloadError) as e:
        warn(e)

    pkg['resources'] = resources

    c.action.package_update(**pkg)

    pkg = c.action.package_show(name_or_id=pkg['id'])

    ##
    ## Add a term with CKAN info.

    doc['Root'].get_or_new_term('CkanId').value = pkg['id']

    if org_name_slug is None and pkg.get('organization'):
        doc['Root'].get_or_new_term('CkanOrg', (pkg.get('organization') or {}).get('name'))

    groups = doc['Root'].find('Group')
    for g in groups:
        doc.remove_term(g)

    for group in pkg.get('groups', []):
        doc['Root'].new_term('Group', group['name'])

    write_doc(doc)


def configure_ckan(m):
    """Load groups and organizations, from a file in Metatab format"""
    from ckanapi import RemoteCKAN

    try:
        doc = MetapackDoc(m.mt_file, cache=m.cache)
    except (IOError, MetatabError) as e:
        err("Failed to open metatab '{}': {}".format(m.mt_file, e))

    c = RemoteCKAN(m.ckan_url, apikey=m.api_key)

    groups = { g['name']:g for g in c.action.group_list(all_fields=True) }

    for g in doc['Groups']:

        if g.value not in groups:
            prt('Creating group: ', g.value)
            c.action.group_create(name=g.value,
                                  title=g.get_value('title'),
                                  description=g.get_value('description'),
                                  id=g.get_value('id'),
                                  image_url=g.get_value('image_url'))

    orgs = {o['name']: o for o in c.action.organization_list(all_fields=True)}

    for o in doc['Organizations']:

        if o.value not in orgs:
            prt('Creating organization: ', o.value)
            c.action.organization_create(name=o.value,
                                  title=o.get_value('title'),
                                  description=o.get_value('description'),
                                  id=o.get_value('id'),
                                  image_url=o.get_value('image_url'))

def load_instructions_description(desc,url):
    """Display loading the package into metapack, displayed in the resource"""

    return dedent(
        """
        {desc}

        Load dataset into a Python program:

            import metapack as mp
            pkg = mp.open_package('{url}')

        """
    ).format(desc=desc, url=url)

def package_load_instructions(inst_distributions):
    """Load instructions, displayed in the package notes"""

    per_package_inst = ''

    for dist in inst_distributions:

        if dist.type == 'zip':
            per_package_inst += dedent(
                """
                # Loading the ZIP Package

                Zip packages are compressed, so large resources may load faster.

                    import metapack as mp
                    pkg = mp.open_package('{url}')

                """.format(url=dist.package_url.inner))

        elif dist.type == 'csv':
            per_package_inst += dedent(
                """
                # Loading the CSV Package

                CSV packages load resources individually, so small resources may load faster.


                    import metapack as mp
                    pkg = mp.open_package('{url}')

                """.format(url=dist.package_url.inner))

    if per_package_inst:

        return '\n---\n'+per_package_inst

    else:
        return ''

def dump_ckan(m):
    """Create a groups and organization file"""

    doc = MetapackDoc(cache=m.cache)
    doc.new_section('Groups',        'Title Description Id Image_url'.split())
    doc.new_section('Organizations', 'Title Description Id Image_url'.split())

    c = RemoteCKAN(m.ckan_url, apikey=m.api_key)

    for g in c.action.group_list(all_fields=True):
        print(g.keys())

    for o in c.action.organization_list(all_fields=True):
        print(g.keys())



# Copyright (c) 2019 Civic Knowledge. This file is licensed under the terms of the
# MIT License, included in this distribution as LICENSE

"""
CLI program for managing packages
"""

import json
from functools import lru_cache
from os.path import basename
from textwrap import dedent
from uuid import uuid4

import yaml
from metapack import Downloader, is_metapack_url
from metapack.cli.core import (MetapackCliMemo, err, warn, get_config, prt)
from metapack.html import display_context, jsonld
from metapack.util import ensure_dir
from metapack_jupyter.wordpress import convert_wordpress
from rowgenerators import parse_app_url

from metapack_build.build import find_csv_packages

downloader = Downloader.get_instance()


def wp(subparsers):
    parser = subparsers.add_parser(
        'wp',
        help='Publish a Jupyter notebook or data package to Wordpress ',
        description="""
        Write a notebook or package to Wordpress.

        If the source argument is a data package, (Metatab format Metapack package, CSV or ZIP )package information will be 
        submitted to the Wordpress blog as a blog post, formatted similarly to the package html documentation file.

        If the source is a Metatab file and the ``--references`` option is set, the references References section of the 
        file are URLs for the packages that should be published. The ``--group`` and ``--tag`` options are used
        for publishing each of the references. 
        

        If the source argument is a Jupyter notebook, the notebook is written as a blog post. See
        http://insights.civicknowledge.com for examples.

        """,
        epilog='Cache dir: {}\n'.format(str(downloader.cache.getsyspath('/'))))

    parser.set_defaults(run_command=run_wp)

    parser.add_argument('-g', '--group', nargs='?', action='append', help="Assign an additional group. Can be used "
                                                                          "multiple times, one onece with values "
                                                                          "separated by commas")

    parser.add_argument('-t', '--tag', nargs='?', action='append', help="Assign an additional tag. Can be used "
                                                                        "multiple times, or once with values "
                                                                        "separated by commas")

    parser.add_argument('-H', '--header', help='Dump YAML of notebook header', action='store_true')

    parser.add_argument('-G', '--get', help='Get and dump the post, rather that update it', action='store_true')

    parser.add_argument('-P', '--profile', help="Name of a BOTO or AWS credentails profile", required=False)

    parser.add_argument('-p', '--publish', help='Set the post state to published, rather than draft',
                        action='store_true')

    parser.add_argument('-R', '--references', help='The metatab package lists the packages to publish as references',
                        action='store_true')

    parser.add_argument('-r', '--result', action='store_true', default=False,
                        help="If mp -q flag set, still report results")

    parser.add_argument('-d', '--dump', help="Dump the HTML. Default if site_name is not specified",
                        action='store_true')

    parser.add_argument('-s', '--site_name', help="Site name, in the .metapack.yaml configuration file")

    parser.add_argument('-T', '--template', help="Name of Jinja2 template", default='wordpress.html')

    parser.add_argument('-n', '--no-op', help='Do everything but submit the post', action='store_true')

    parser.add_argument('source',nargs='?', help="Path to a notebook file or a Metapack package")


def split_groups_tags(v):
    if not v:
        return

    for g in v:
        if ',' in g:
            for g_ in g.split(','):
                yield g_
        else:
            yield g

def run_wp(args):

    if not args.source:
        args.source = '.'

    u = parse_app_url(args.source)

    if u.target_format == 'ipynb':
        run_notebook(args)
    else:

        args.metatabfile = args.source
        m = MetapackCliMemo(args, downloader)

        if args.dump or not args.site_name:
            doc, content = get_doc_content(m)
            print(content)
        elif args.site_name:

            if args.references:

                m.args.group = m.args.group if m.args.group else []
                m.args.group += [e.value for e in m.doc.find('Root.Group')]

                m.args.tag = m.args.tag if m.args.tag else []
                m.args.tag += [e.value for e in m.doc.find('Root.Tag')]

                for r in m.doc.references():

                    u = r.resolved_url
                    if is_metapack_url(u):
                        args.metatabfile = str(u)
                        mc = MetapackCliMemo(m.args, downloader)

                        if args.result:
                            print(f" ✅ Publishing {str(u)} to {args.site_name}", )
                        else:
                            prt(f"Publishing {str(u)} to {args.site_name}", str(u))

                        run_package(mc)

            else:
                run_package(m)
                if args.result:
                    print(f"✅ Published {str(m.doc.name)} to {args.site_name}", )
                else:
                    prt(f"Published {str(m.doc.name)} to {args.site_name}", str(u))


def run_notebook(args):
    p = '/tmp/metapack-wp-notebook/'
    ensure_dir(p)

    output_file, resources = convert_wordpress(args.source, p)

    r, post = publish_wp(args.site_name, output_file, resources, args)
    prt("Post url: ", post.link)


def get_site_config(site_name):
    config = get_config()

    if config is None:
        err("No metatab configuration found. Can't get Wordpress credentials. Maybe create '~/.metapack.yaml'")

    site_config = config.get('wordpress', {}).get(site_name, {})

    if not site_config:
        err("In config file '{}', expected 'wordpress.{}' section for site config"
            .format(config['_loaded_from'], site_name))

    if 'url' not in site_config or 'user' not in site_config or 'password' not in site_config:
        err(dedent(
            """
            Incomplete configuration. Expected:
                wordpress.{site_name}.url
                wordpress.{site_name}.user
                wordpress.{site_name}.password
            In configuration file '{cfg_file}'
            """.format(site_name=site_name, cfg_file=config['_loaded_from'])
        ))

    return site_config['url'], site_config['user'], site_config['password']


def prepare_image(slug, file_name, post_id):
    from xmlrpc import client as xmlrpc_client

    # prepare metadata
    data = {
        'name': 'picture.jpg',
        'type': 'image/jpeg',  # mimetype
    }

    # read the binary file and let the XMLRPC library encode it into base64
    with open(file_name, 'rb') as img:
        return {
            'name': '{}-{}'.format(slug, basename(file_name)),
            'type': 'image/png',  # mimetype
            'bits': xmlrpc_client.Binary(img.read()),
            'post_id': post_id
        }


def cust_field_dict(post):
    try:
        return dict((e['key'], e['value']) for e in post.custom_fields)
    except (KeyError, AttributeError):
        return {}


@lru_cache()
def get_posts(wp):
    """Get and memoize all posts"""
    from wordpress_xmlrpc.methods.posts import GetPosts

    all_posts = []

    offset = 0
    increment = 20
    while True:
        posts = wp.call(GetPosts({'number': increment, 'offset': offset}))
        if len(posts) == 0:
            break  # no more posts returned
        for post in posts:
            all_posts.append(post)

        offset = offset + increment

    return all_posts


def find_post(wp, identifier):
    for _post in get_posts(wp):
        if cust_field_dict(_post).get('identifier') == identifier:
            return _post

    return None


def set_custom_field(post, key, value):
    if not hasattr(post, 'custom_fields') or not key in [e['key'] for e in post.custom_fields]:

        if not hasattr(post, 'custom_fields'):
            post.custom_fields = []

        post.custom_fields.append({'key': key, 'value': value})


def publish_wp(site_name, output_file, resources, args):
    """Publish a notebook to a wordpress post, using Gutenberg blocks.

    Here is what the metadata looks like, in a section of the notebook tagged 'frontmatter'

    show_input: hide
    github: https://github.com/sandiegodata/notebooks/blob/master/tutorial/American%20Community%20Survey.ipynb
    identifier: 5c987397-a954-46ca-8743-bdcd7a71579c
    featured_image: 171
    authors:
    - email: eric@civicknowledge.com
      name: Eric Busboom
      organization: Civic Knowledge
      type: wrangler
    tags:
    - Tag1
    - Tag2
    categories:
    - Demographics
    - Tutorial

    'Featured_image' is an attachment id

    """

    from wordpress_xmlrpc import Client, WordPressPost
    from wordpress_xmlrpc.methods.media import UploadFile, GetMediaLibrary
    from wordpress_xmlrpc.methods.posts import NewPost, EditPost, GetPost
    from xmlrpc.client import Fault

    url, user, password = get_site_config(site_name)

    meta = {}
    for r in resources:
        if r.endswith('.json'):
            with open(r) as f:
                meta = json.load(f)

    fm = meta.get('frontmatter', {})

    if not 'identifier' in fm or not fm['identifier']:
        err("Can't publish notebook without a unique identifier. Add this to the "
            "Metatab document or frontmatter metadata:\n   identifier: {}".format(str(uuid4())))

    wp = Client(url, user, password)

    post = find_post(wp, fm['identifier'])

    if post:
        prt("Updating old post")
    else:
        post = WordPressPost()
        post.id = wp.call(NewPost(post))
        prt(f"Creating new post; could not find identifier {fm['identifier']} ")

    post.title = fm.get('title', '')
    post.slug = fm.get('slug')

    with open(output_file) as f:
        content = f.read()

    post.terms_names = {
        'post_tag': fm.get('tags', []),
        'category': fm.get('categories', [])
    }

    if args.header:
        print(yaml.dump(fm, default_flow_style=False))

    set_custom_field(post, 'identifier', fm['identifier'])

    post.excerpt = fm.get('excerpt', fm.get('brief', fm.get('description')))

    def strip_image_name(n):
        """Strip off the version number from the media file"""
        from os.path import splitext
        import re
        return re.sub(r'\-\d+$', '', splitext(n)[0])

    extant_files = list(wp.call(GetMediaLibrary(dict(parent_id=post.id))))

    def find_extant_image(image_name):
        for img in extant_files:
            if strip_image_name(basename(img.metadata['file'])) == strip_image_name(image_name):
                return img

        return None

    for r in resources:

        image_data = prepare_image(fm['identifier'], r, post.id)
        img_from = "/{}/{}".format(fm['slug'], basename(r))

        extant_image = find_extant_image(image_data['name'])

        if extant_image:
            prt("Post already has image:", extant_image.id, extant_image.link)
            img_to = extant_image.link

        elif r.endswith('.png'):  # Foolishly assuming all images are PNGs
            if args.no_op:
                response = {
                    'id': None,
                    'link': 'http://example.com'
                }
            else:
                response = wp.call(UploadFile(image_data, overwrite=True))

            prt("Uploaded image {} to id={}, {}".format(basename(r), response['id'], response['link']))

            img_to = response['link']

        else:
            continue



        content = content.replace(img_from, img_to)

    if fm.get('featured_image') and str(fm.get('featured_image')).strip():
        post.thumbnail = int(fm['featured_image'])
    elif hasattr(post, 'thumbnail') and isinstance(post.thumbnail, dict):
        # The thumbnail expects an attachment id on EditPost, but returns a dict on GetPost
        post.thumbnail = post.thumbnail['attachment_id']

    post.content = content

    if not args.no_op:
        try:
            r = wp.call(EditPost(post.id, post))
        except Fault as e:
            if 'attachment' in str(e): # Remove attachment and try again.
                post.thumbnail = None
                r = wp.call(EditPost(post.id, post))


        return r, wp.call(GetPost(post.id))


def html(doc, template):
    from markdown import markdown as convert_markdown
    from bs4 import BeautifulSoup
    from jinja2 import Environment, PackageLoader

    context = display_context(doc)

    if 'inline_doc' in context:
        extensions = [
            'markdown.extensions.extra',
            'markdown.extensions.admonition'
        ]

        inline_doc = convert_markdown(context['inline_doc'], extensions)

        idp = []

        tag_map = {
            'p': 'wp:paragraph',
            'ul': 'wp:list',
            'pre': 'wp:code',
            'h1': 'wp:heading',
            'h2': 'wp:heading',
            'h3': 'wp:heading',
            'h4': 'wp:heading',
            'h5': 'wp:heading',
            'h6': 'wp:heading',
        }

        nodes = BeautifulSoup(inline_doc, features="html.parser").find_all(recursive=False)

        context['inline_doc_parts'] = \
            [(tag_map.get(n.name, 'wp:html'), str(n)) for n in nodes]

    else:
        context['inline_doc_parts'] = []

    context['jsonld'] = json.dumps(jsonld(doc), indent=4)

    #print(json.dumps(context, indent=4))

    env = Environment(
        loader=PackageLoader('metapack_wp', 'support/templates')
        # autoescape=select_autoescape(['html', 'xml'])
    )

    return env.get_template(template).render(**context)


def get_doc_content(m):
    if not m.doc.find('Root.Distribution'):

        doc = find_csv_packages(m, downloader)
        if not doc:
            err('Package has no Root.Distribution, and no CSV package found')
            doc = m.doc
        else:
            prt("Package has no Root.Distribution, using CSV package: ", doc.ref)
    else:
        doc = m.doc

    return doc, html(doc, m.args.template)


def run_package(m):
    """Publish documentation for a package as a post"""
    from wordpress_xmlrpc import Client, WordPressPost
    from xmlrpc.client import Fault
    from wordpress_xmlrpc.methods.posts import NewPost, EditPost
    from slugify import slugify

    doc, content = get_doc_content(m)

    url, user, password = get_site_config(m.args.site_name)
    wp = Client(url, user, password)

    post = find_post(wp, doc.identifier)

    if m.args.get:
        if post:
            post.content = "<content elided>"
            from pprint import pprint
            pprint(post.struct)
            return
        else:
            warn(f"Didn't find post for identifier {doc.identifier}")
            return

    if post:
        prt("Updating old post")
        action = lambda post: EditPost(post.id, post)
    else:
        prt(f"Creating new post; could not find identifier '{doc.identifier}' ")
        print(doc.identifier)
        action = lambda post: NewPost(post)
        post = WordPressPost()

    set_custom_field(post, 'identifier', doc.identifier)
    set_custom_field(post, 'name', doc.name)
    set_custom_field(post, 'nvname', doc.nonver_name)

    post.excerpt = doc['Root'].get_value('Root.Description') or content[:200]

    post_tags = list(set(
        [t.value for t in doc['Root'].find('Root.Tag')] +
        [t.value for t in doc['Root'].find('Root.Group')] +
        [doc['Root'].get_value('Root.Origin')] +
        list(split_groups_tags(m.args.group)) +
        list(split_groups_tags(m.args.tag))
    ))

    post.terms_names = {
        'post_tag': post_tags,
        'category': ['Dataset'] + list(split_groups_tags(m.args.group))
    }

    post.title = doc.get_value('Root.Title')
    post.slug = slugify(doc.nonver_name)
    post.content = content

    if m.args.publish:
        post.post_status = 'publish'

    try:
        if m.args.no_op:
            r = {}
        else:
            r = wp.call(action(post))
    except Fault as e:

        if 'taxonomies' in e.faultString:
            err(("User {} does not have permissions to add terms to taxonomies. "
                 "Terms are: {}").format(user, post.terms_names))

        raise

    return r
